@@ -15,12 +15,9 @@ Functions
 
     cases
     deaths
-    R
+    nfc
     npi
     npis
-    hospitalization
-    icu
-    vaccinations
     reports
     timeseries
 """
@@ -29,13 +26,11 @@ from pycocap import pycocap
 from pycocap.pycocap import (
     cases,
     deaths,
-    hospitalization,
-    icu,
+    nfc,
     npi,
     npis,
-    R,
     reports,
-    vaccinations,
+    timeseries,
 )
 
 try:
@@ -48,11 +43,9 @@ __all__ = ['__version__']
 __all__ += [
     'cases',
     'deaths',
-    'hospitalization',
-    'icu',
+    'nfc',
     'npi',
     'npis',
-    'R',
     'reports',
-    'vaccinations',
+    'timeseries',
 ]
