@@ -3,7 +3,7 @@
 import warnings
 from functools import reduce
 from datetime import date
-from typing import Union
+from typing import Any, Optional, Union
 import requests
 import numpy as np
 import pandas as pd
@@ -14,8 +14,8 @@ API_URL = 'https://webapp.ufz.de/cocap/'
 
 def cut_at_dates(
     df: pd.DataFrame,
-    start_date: date = None,
-    end_date: date = None,
+    start_date: Optional[date] = None,
+    end_date: Optional[date] = None,
 ) -> pd.DataFrame:
     """Cut a DataFrame at given dates."""
     if (start_date is not None or end_date is not None) and start_date == end_date:
@@ -36,9 +36,9 @@ def cut_at_dates(
 
 def regularize_timeseries(
     df: pd.DataFrame,
-    fill_value: float = None,
-    start_date: date = None,
-    end_date: date = None,
+    fill_value: Optional[float] = None,
+    start_date: Optional[date] = None,
+    end_date: Optional[date] = None,
 ) -> pd.DataFrame:
     """Fill missing data with NaNs, or fill_value to get a regular timeseries"""
     # TODO can all this be done cleaner?!
@@ -65,14 +65,15 @@ def regularize_timeseries(
 
 def _convert_json_to_df(
     json,
-    rename_cols: dict = None,
-    start_date: date = None,
-    end_date: date = None,
+    rename_cols: Optional[dict] = None,
+    start_date: Optional[date] = None,
+    end_date: Optional[date] = None,
+    date_format: Optional[str] = None,
 ) -> pd.DataFrame:
     df = pd.DataFrame(json)
     if rename_cols is not None:
         df = df.rename(columns=rename_cols)
-    df['date'] = pd.to_datetime(df['date'])
+    df['date'] = pd.to_datetime(df['date'], format=date_format)
     df = cut_at_dates(df, start_date, end_date)
     return df
 
@@ -80,12 +81,12 @@ def _convert_json_to_df(
 def reports(
     country: str,
     report_type: str,
-    start_date: date = None,
-    end_date: date = None,
-    params: dict = None,
-    fill_value=np.nan,
+    start_date: Optional[date] = None,
+    end_date: Optional[date] = None,
+    params: Optional[dict] = None,
+    fill_value: Union[float, str] = np.nan,
     return_population: bool = False,
-):
+) -> Any:
     """Get cases or deaths timeseries as a DataFrame from the db.
 
     Parameters
@@ -132,7 +133,7 @@ def reports(
         if fill_value != 'drop':
             df = regularize_timeseries(
                 df,
-                fill_value,
+                fill_value,  # type: ignore
                 start_date,
                 end_date,
             )
@@ -147,12 +148,12 @@ def reports(
 
 def cases(
     country: str,
-    start_date: date = None,
-    end_date: date = None,
-    params: dict = None,
+    start_date: Optional[date] = None,
+    end_date: Optional[date] = None,
+    params: Optional[dict] = None,
     fill_value=np.nan,
     return_population: bool = False,
-):
+) -> Any :
     """Get cases timeseries as a DataFrame from the db.
 
     Parameters
@@ -198,12 +199,12 @@ def cases(
 
 def deaths(
     country: str,
-    start_date: date = None,
-    end_date: date = None,
-    params: dict = None,
+    start_date: Optional[date] = None,
+    end_date: Optional[date] = None,
+    params: Optional[dict] = None,
     fill_value=np.nan,
     return_population: bool = False,
-):
+) -> Any:
     """Get death timeseries as a DataFrame from the db.
 
     Parameters
@@ -250,10 +251,10 @@ def deaths(
 def timeseries(
     country: str,
     report_type: str,
-    start_date: date = None,
-    end_date: date = None,
-    params: dict = None,
-    fill_value=np.nan,
+    start_date: Optional[date] = None,
+    end_date: Optional[date] = None,
+    params: Optional[dict] = None,
+    fill_value: Union[float, str] = np.nan,
 ) -> pd.DataFrame:
     """Get a timeseries as a DataFrame from the db.
 
@@ -264,10 +265,10 @@ def timeseries(
     report_type
         Can be
 
-        * "hospital_daily"
-        * "hospital_weekly"
-        * "icu_daily"
-        * "icu_weekly"
+        * "hospital_daily_occupation"
+        * "hospital_weekly_admissions"
+        * "icu_daily_occupation"
+        * "icu_weekly_admission"
         * "R"
         * "vaccinations_daily"
         * "vaccinations_people"
@@ -297,7 +298,21 @@ def timeseries(
     df
         A DataFrame consisting of the columns "date" and the given timeseries
     """
-    response = requests.get(API_URL + f'{country}/{report_type}', params=params)
+    valid_types = (
+        'hospital_daily_occupation',
+        'hospital_weekly_admissions',
+        'icu_daily_occupation',
+        'icu_weekly_admissions',
+        'R',
+        'vaccinations_daily',
+        'vaccinations_people',
+        'vaccinations_peoplefully',
+        'vaccinations_total',
+        'vaccinations_total_booster',
+    )
+    if report_type is not None and report_type not in valid_types:
+        raise ValueError(f'report_type can only be one of {valid_types}')
+    response = requests.get(API_URL + f'{country}/timeseries/{report_type}', params=params)
     req_name = 'timeseries'
     if response.status_code == 200:
         r_json = response.json()
@@ -313,7 +328,7 @@ def timeseries(
         if fill_value != 'drop':
             df = regularize_timeseries(
                 df,
-                fill_value,
+                fill_value,  # type: ignore
                 start_date,
                 end_date,
             )
@@ -322,26 +337,39 @@ def timeseries(
         df = None
     return df
 
-def hospitalization(
+def nfc(
     country: str,
-    report_type: str = 'all',
-    start_date: date = None,
-    end_date: date = None,
-    params: dict = None,
-    fill_value=np.nan,
+    nfc_type: str,
+    start_date: Optional[date] = None,
+    end_date: Optional[date] = None,
+    params: Optional[dict] = None,
 ) -> pd.DataFrame:
-    """Get hospitalization data as a DataFrame from the db.
+    """Get National Framework Conditions as a DataFrame from db.
 
     Parameters
     ----------
     country
         The country name for which to get the timeseries
-    report_type : optional
-        the type of data, if `None`, returns all. Can be
+    nfc_type
+        the type of data
 
-        * None (default)
-        * "hospital_daily"
-        * "hospital_weekly"
+        * "age"
+        * "air_pollution_pm2.5"
+        * "gender_equality"
+        * "health_expenditure"
+        * "immunization_dpt"
+        * "life_expectancy_at_birth"
+        * "literacy_rate_adult"
+        * "malaria"
+        * "obesity"
+        * "open_defaction"
+        * "population"
+        * "population_density"
+        * "rtd_expenditure"
+        * "social_insurance_coverage"
+        * "tobacco_use"
+        * "tourist_departures"
+        * "urban_population"
 
     start_date : optional
         The date at which the timeseries will start
@@ -352,265 +380,45 @@ def hospitalization(
 
         * "source_name"
 
-    fill_value : optional
-        decide what to do with missing values, can be
-
-        * np.nan: fill with NaNs, to optain regular timeseries
-        * a number: fill with constant value, e.g. 0
-        * 'drop': drop missing values
-        * 'interpolate' : linear interpolation over gaps
-
     Returns
     -------
     df
-        A DataFrame consisting of the columns "date" and the report types
+        A DataFrame consisting of the columns "date" and `nfc_type`
     """
-    valid_types = ('hospital_daily', 'hospital_weekly')
-    if report_type is not None and report_type not in valid_types:
-        raise ValueError(f'report_type can only be one of {valid_types}')
-    df = None
-    if report_type is None:
-        df = timeseries(
-            country,
-            'hospital_daily',
+    response = requests.get(API_URL + f'{country}/nfc/{nfc_type}', params=params)
+    if response.status_code == 200:
+        r_json = response.json()
+        if len(r_json['nfc']) == 0:
+            warnings.warn('Empty response')
+            return None
+        df = _convert_json_to_df(
+            r_json['nfc'],
+            {'year': 'date', 'value': nfc_type},
             start_date,
             end_date,
-            params,
-            fill_value,
+            date_format='%Y',
         )
-        df_weekly = timeseries(
-            country,
-            'hospital_weekly',
-            start_date,
-            end_date,
-            params,
-            fill_value,
-        )
-        df['hospital_weekly'] = df_weekly['hospital_weekly']
+        # TODO implement regularization for yearly timesteps
+        # if fill_value != 'drop':
+            # df = regularize_timeseries(
+                # df,
+                # fill_value,
+                # start_date,
+                # end_date,
+                # frequency='Y',
+            # )
     else:
-        df = timeseries(
-            country,
-            report_type,
-            start_date,
-            end_date,
-            params,
-            fill_value,
-        )
-    return df
-
-def icu(
-    country: str,
-    report_type: str = 'all',
-    start_date: date = None,
-    end_date: date = None,
-    params: dict = None,
-    fill_value=np.nan,
-) -> pd.DataFrame:
-    """Get ICU data as a DataFrame from the db.
-
-    Parameters
-    ----------
-    country
-        The country name for which to get the timeseries
-    report_type : optional
-        the type of data, if `None`, returns all. Can be
-
-        * None (default)
-        * "icu_daily"
-        * "icu_weekly"
-
-    start_date : optional
-        The date at which the timeseries will start
-    end_date : optional
-        The date at which the timeseries will end
-    params : optional
-        optional query parameters
-
-        * "source_name"
-
-    fill_value : optional
-        decide what to do with missing values, can be
-
-        * np.nan: fill with NaNs, to optain regular timeseries
-        * a number: fill with constant value, e.g. 0
-        * 'drop': drop missing values
-        * 'interpolate' : linear interpolation over gaps
-
-    Returns
-    -------
-    df
-        A DataFrame consisting of the columns "date" and the report types
-    """
-    valid_types = ('icu_daily', 'icu_weekly')
-    if report_type is not None and report_type not in valid_types:
-        raise ValueError(f'report_type can only be one of {valid_types}')
-    df = None
-    if report_type is None:
-        df = timeseries(
-            country,
-            'icu_daily',
-            start_date,
-            end_date,
-            params,
-            fill_value,
-        )
-        df_weekly = timeseries(
-            country,
-            'icu_weekly',
-            start_date,
-            end_date,
-            params,
-            fill_value,
-        )
-        df['icu_weekly'] = df_weekly['icu_weekly']
-    else:
-        df = timeseries(
-            country,
-            report_type,
-            start_date,
-            end_date,
-            params,
-            fill_value,
-        )
-    return df
-
-def R(
-    country: str,
-    start_date: date = None,
-    end_date: date = None,
-    params: dict = None,
-    fill_value=np.nan,
-) -> pd.DataFrame:
-    """Get Reproduction value R as a DataFrame from the db.
-
-    Parameters
-    ----------
-    country
-        The country name for which to get the timeseries
-    start_date : optional
-        The date at which the timeseries will start
-    end_date : optional
-        The date at which the timeseries will end
-    params : optional
-        optional query parameters
-
-        * "source_name"
-
-    fill_value : optional
-        decide what to do with missing values, can be
-
-        * np.nan: fill with NaNs, to optain regular timeseries
-        * a number: fill with constant value, e.g. 0
-        * 'drop': drop missing values
-        * 'interpolate' : linear interpolation over gaps
-
-    Returns
-    -------
-    df
-        A DataFrame consisting of the columns "date" and "R"
-    """
-    return timeseries(
-        country,
-        'R',
-        start_date,
-        end_date,
-        params,
-        fill_value,
-    )
-
-def vaccinations(
-    country: str,
-    report_type: str = None,
-    start_date: date = None,
-    end_date: date = None,
-    params: dict = None,
-    fill_value=np.nan,
-) -> pd.DataFrame:
-    """Get daily vaccinations as a DataFrame from the db.
-
-    Parameters
-    ----------
-    country
-        The country name for which to get the timeseries
-    report_type : optional
-        the type of data, if `None`, returns all. Can be
-
-        * None (default)
-        * "vaccinations_daily"
-        * "vaccinations_people"
-        * "vaccinations_people_fully"
-        * "vaccinations_total"
-        * "vaccinations_total_booster"
-
-    start_date : optional
-        The date at which the timeseries will start
-    end_date : optional
-        The date at which the timeseries will end
-    params : optional
-        optional query parameters
-
-        * "source_name"
-
-    fill_value : optional
-        decide what to do with missing values, can be
-
-        * np.nan: fill with NaNs, to optain regular timeseries
-        * a number: fill with constant value, e.g. 0
-        * 'drop': drop missing values
-        * 'interpolate' : linear interpolation over gaps
-
-    Returns
-    -------
-    df
-        A DataFrame consisting of the columns "date" and "vaccinations_daily"
-    """
-    valid_types = (
-        'vaccinations_daily',
-        'vaccinations_people',
-        'vaccinations_people_fully',
-        'vaccinations_total',
-        'vaccinations_total_booster',
-    )
-    if report_type is not None and report_type not in valid_types:
-        raise ValueError(f'report_type can only be one of {valid_types}')
-    df = None
-    if report_type is None:
-        df = timeseries(
-            country,
-            valid_types[1],
-            start_date,
-            end_date,
-            params,
-            fill_value,
-        )
-        for r in valid_types[1:]:
-            df_app = timeseries(
-                country,
-                r,
-                start_date,
-                end_date,
-                params,
-                fill_value,
-            )
-            df[r] = df_app[r]
-    else:
-        df = timeseries(
-            country,
-            report_type,
-            start_date,
-            end_date,
-            params,
-            fill_value,
-        )
+        print(f'Response code: {response.status_code}')
+        df = None
     return df
 
 def npi(
     country: str,
     npi_type: str,
-    start_date: date = None,
-    end_date: date = None,
-    params: dict = None,
-    fill_value=np.nan,
+    start_date: Optional[date] = None,
+    end_date: Optional[date] = None,
+    params: Optional[dict] = None,
+    fill_value: Union[float, str] = np.nan,
 ) -> pd.DataFrame:
     """Get NPI timeseries as a DataFrame from the db.
 
@@ -676,7 +484,7 @@ def npi(
         if fill_value != 'drop':
             df = regularize_timeseries(
                 df,
-                fill_value,
+                fill_value,  # type: ignore
                 start_date,
                 end_date,
             )
@@ -687,11 +495,11 @@ def npi(
 
 def npis(
     country: str,
-    npi_types: Union[list, str] = None,
-    start_date: date = None,
-    end_date: date = None,
-    params: dict = None,
-    fill_value=np.nan,
+    npi_types: Optional[Union[list, str]] = None,
+    start_date: Optional[date] = None,
+    end_date: Optional[date] = None,
+    params: Optional[dict] = None,
+    fill_value: Union[float, str] = np.nan,
 ) -> pd.DataFrame:
     """Get multiple NPI timeseries as a DataFrame from the db.
 
@@ -773,7 +581,7 @@ def npis(
             end_date,
             params,
             fill_value,
-        ) for npi_type in _npi_types
+            ) for npi_type in _npi_types  # pyright: ignore
     ]
     dfs = reduce(lambda df1, df2: pd.merge(df1, df2, on='date'), dfs)
     return dfs
